/*
**  Copyright 2000-2004 University of Illinois Board of Trustees
**  Copyright 2000-2004 Mark D. Roth
**  All rights reserved.
**
**  du.c - FTP disk usage code
**
**  Mark D. Roth <roth@feep.net>
*/

#include <fget.h>

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>

#ifdef STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#endif

#ifdef HAVE_INTTYPES_H
# include <inttypes.h>
#endif


static int
fget_du_file(FTP *ftp, char *filename, struct ftpstat *fsp, void *total)
{
	uint64_t my_total = 0;

#ifdef DEBUG
	printf("==> fget_du_file(filename=\"%s\", total=%lu)\n",
	       filename, *(unsigned long *)total);
#endif

#ifdef DEBUG
	ftp_long_ls(ftp, filename, fsp);
#endif
	*((uint64_t *)total) += fsp->fs_size;

	if (BIT_ISSET(options, OPT_RECURSIVE) && S_ISDIR(fsp->fs_mode))
	{
		if (fget_recursion(ftp, filename, fget_du_file,
				   &my_total) == R_RETURN)
			return R_RETURN;

		printf("\t%10lld  %s\n", (my_total / 1024), filename);
		*((uint64_t *)total) += my_total;
	}

	return R_FILEOK;
}


int
fget_du(FTP *ftp, char *filename)
{
	uint64_t total = 0;
	struct ftpstat fs;
	int retval;
	fget_listptr_t lp1, lp2;

	putchar('\n');

	if (BIT_ISSET(options, OPT_CHASELINKS) && chase_l == NULL)
	{
		chase_l = fget_list_new(LIST_QUEUE, NULL);
		done_files_l = fget_list_new(LIST_QUEUE, NULL);
		if (chase_l == NULL || done_files_l == NULL)
		{
			fprintf(stderr, "fget: fget_list_new(): %s\n",
				strerror(errno));
			return R_RETURN;
		}
	}

	if (fget_stat(ftp, filename, &fs) != R_FILEOK)
		return R_RETURN;

	retval = fget_du_file(ftp, filename, &fs, &total);

	if (chase_l != NULL && done_files_l != NULL &&
	    BIT_ISSET(options, OPT_CHASELINKS))
	{
		fget_listptr_reset(&lp1);
		while (fget_list_next(chase_l, &lp1) != 0 && retval != R_RETURN)
		{
			char *cp;

			cp = (char *)fget_listptr_data(&lp1);

			fget_listptr_reset(&lp2);
			if (fget_list_search(done_files_l, &lp2, cp,
					     (BIT_ISSET(options, OPT_RECURSIVE)
					      ? (fget_matchfunc_t)subdir_match
					      : NULL)) == 0)
			{
				retval = fget_stat(ftp, cp, &fs);
				if (retval == R_FILEOK)
					retval = fget_du_file(ftp, cp,
							      &fs, &total);
			}
		}
		fget_list_free(chase_l, free);
		fget_list_free(done_files_l, free);
		chase_l = NULL;
		done_files_l = NULL;
	}

	if (retval == R_FILEOK)
	{
		printf("\t%10lld  %s\n\n", (total / 1024), filename);
		printf("fget: total is %lld Kbytes\n", (total / 1024));
	}

	return retval;
}


