/*
**  Copyright 2000-2004 University of Illinois Board of Trustees
**  Copyright 2000-2004 Mark D. Roth
**  All rights reserved.
**
**  list.c - FTP file listing code for fget
**
**  Mark D. Roth <roth@feep.net>
*/

#include <fget.h>

#include <stdio.h>
#include <errno.h>

#ifdef STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#endif


static int
fget_list_file(FTP *ftp, char *filename, struct ftpstat *fsp, void *dummy)
{
#ifdef DEBUG
	printf("==> fget_list_file(filename=\"%s\")\n", filename);
#endif

	ftp_long_ls(ftp, filename, fsp);

	if (BIT_ISSET(options, OPT_RECURSIVE) && S_ISDIR(fsp->fs_mode)
	    && chase_l != NULL)
	{
#ifdef DEBUG
		printf("    fget_list_file(): adding %s to chase_l\n",
		       filename);
#endif
		fget_list_add(chase_l, strdup(filename));
	}

	return R_FILEOK;
}


int
fget_list(FTP* ftp, char *filename)
{
	fget_listptr_t lp;
	struct ftpstat fs;
	int retval, freeflag = 0;

#ifdef DEBUG
	printf("==> fget_list(\"%s\")\n", filename);
#endif

	if (chase_l == NULL &&
	    (BIT_ISSET(options, OPT_RECURSIVE) ||
	     BIT_ISSET(options, OPT_CHASELINKS)))
	{
		chase_l = fget_list_new(LIST_QUEUE, NULL);
		if (chase_l == NULL)
			return R_RETURN;
		if (BIT_ISSET(options, OPT_CHASELINKS))
		{
			done_files_l = fget_list_new(LIST_QUEUE, NULL);
			if (done_files_l == NULL)
			{
				fget_list_free(chase_l, free);
				return R_RETURN;
			}
		}
		freeflag = 1;
	}

	if (fget_stat(ftp, filename, &fs) != R_FILEOK)
		return R_RETURN;

	fget_listptr_reset(&lp);
	if (S_ISLNK(fs.fs_mode)
	    && done_files_l != NULL
	    && fget_list_search(done_files_l, &lp, filename,
				(BIT_ISSET(options, OPT_RECURSIVE)
				? (fget_matchfunc_t)subdir_match : NULL)) != 0)
		return R_SKIP;

	if (verbose)
		printf("\nfget: listing %s...\n", filename);
	if (S_ISDIR(fs.fs_mode))
		retval = fget_recursion(ftp, filename, fget_list_file, NULL);
	else
		retval = fget_list_file(ftp, filename, &fs, NULL);

	if (freeflag &&
	    (BIT_ISSET(options, OPT_RECURSIVE) ||
	     BIT_ISSET(options, OPT_CHASELINKS)))
	{
		fget_listptr_reset(&lp);
		while (fget_list_next(chase_l, &lp) != 0
		       && retval != R_RETURN)
			retval = fget_list(ftp, (char *)fget_listptr_data(&lp));
		fget_list_free(chase_l, free);
		chase_l = NULL;
	}

#ifdef DEBUG
	printf("<== fget_list()\n");
#endif

	return retval;
}


